import { finalize } from 'rxjs/operators';
export class Cycler {
    constructor() {
        this._destructors = [];
        this._transientDestructors = {};
    }
    /**
     * Disposes the Cycler causing all it's dependant managed
     * entities to be disposed. This method will be called
     * automatically by Angular if the Cycler is properly
     * provided.
     */
    dispose() {
        for (let i = 0; i < this._destructors.length; i++) {
            const destructor = this._destructors[i];
            destructor();
        }
        this._destructors = [];
    }
    ngOnDestroy() {
        this.dispose();
    }
    manage(dependency) {
        const disposable = dependency;
        if (disposable.dispose !== undefined) {
            return this.manageDisposable(disposable);
        }
        const subscription = dependency;
        if (subscription.unsubscribe !== undefined) {
            return this.manageSubscription(subscription);
        }
        return this.manageDestructor(dependency);
    }
    manageDestructor(destructor) {
        this._destructors.push(destructor);
        return {
            dispose: () => {
                const destructorIndex = this._destructors.indexOf(destructor);
                if (destructorIndex > 0) {
                    destructor();
                    this._destructors.splice(destructorIndex, 1);
                }
            }
        };
    }
    manageDisposable(disposable) {
        return this.manageDestructor(() => disposable.dispose());
    }
    manageTransient(code, destructor) {
        if (this._transientDestructors[code]) {
            this._transientDestructors[code].dispose();
        }
        const result = this.manageDestructor(destructor);
        this._transientDestructors[code] = result;
        return result;
    }
    manageSubscription(subscription) {
        return this.manageDestructor(() => subscription.unsubscribe());
    }
    manageTransientSubscription(code, subscription) {
        return this.manageTransient(code, () => {
            subscription.unsubscribe();
        });
    }
    manageSubscriptions(...subscriptions) {
        for (const subscription of subscriptions) {
            this.manageSubscription(subscription);
        }
    }
    managedSubscribe(observable, next) {
        let destructor = null;
        const subscription = observable
            .pipe(finalize(() => {
            if (destructor !== null) {
                this._destructors = this._destructors.filter(d => d !== destructor);
            }
        }))
            .subscribe(next);
        destructor = () => subscription.unsubscribe();
        return this.manageDestructor(destructor);
    }
}
//# sourceMappingURL=cycler.js.map